@@ -19,34 +19,64 @@ import pickle
 import re
 import dbus
 import dbus.mainloop.qt
-#PYLINT:C:Comma not followed by a space
 import datetime
 import time
 from nwmanager import NetworkManager
-
-__version__ = '0.0.6'
+import dbus.service
+import dbus.mainloop.qt
+__version__ = '0.0.7'
 AVATAR_CACHE_FOLDER = os.path.join(os.path.expanduser("~"),'.khweeteur','cache')
-CACHE_PATH = os.path.join(os.path.expanduser("~"),'.khweeteur','tweets.cache')
+CACHE_PATH = os.path.join(os.path.expanduser("~"), '.khweeteur','tweets.cache')
 
+class KhweeteurDBus(dbus.service.Object):
+    @dbus.service.method("net.khertan.khweeteur",
+                         in_signature='', out_signature='')
+    def show(self):
+        print 'Show'
+        self.win.activateWindow()
+        
+    def attach_win(self,win):
+        self.win = win
+        
 class KhweeteurNotification(QObject):
     def __init__(self):
         QObject.__init__(self)
         self.m_bus = dbus.SystemBus()
         self.m_notify = self.m_bus.get_object('org.freedesktop.Notifications',
                                               '/org/freedesktop/Notifications')
+        self.iface = dbus.Interface(self.m_notify,'org.freedesktop.Notifications')
+
+    def warn(self,message):
+        self.iface.SystemNoteDialog(message,0,'Nothing')
+        
+    def info(self,message):
+        self.iface.SystemNoteInfoprint('Khweeteur : '+message)
+        
+    def notify(self,title,message,category='im.received',icon='khweeteur_32',count=1):
+        self.iface.Notify('Khweeteur',
+                          0,
+                          icon,
+                          title,
+                          message,
+                          ['default','test'],
+                          {'category':category,
+                          'desktop-entry':'khweeteur',
+                          'dbus-callback-default':'net.khertan.khweeteur /net/khertan/khweeteur net.khertan.khweeteur show',
+                          'count':count},
+                          -1
+                          )
+#        self.m_notify.Notify(title,
+#                             0,
+#                             icon,
+#                             title,
+#                             message,
+#                             [],
+#                             {'category':category,
+#                             'count':count},
+#                             -1,
+#                             dbus_interface='org.freedesktop.Notifications'
+#                             )
 
-    def send(self,title,message,category='message',icon='khweeteur',count=1):
-        self.m_notify.Notify(title,
-                             0,
-                             icon,
-                             title,
-                             message,
-                             [],
-                             {'category':category,
-                             'count':count},
-                             -1,
-                             dbus_interface='org.freedesktop.Notifications'
-                             )
 
 class KhweeteurWorker(QThread):
 
@@ -66,7 +96,6 @@ class KhweeteurWorker(QThread):
         
     def downloadProfileImage(self,status):
         if type(status)!=twitter.DirectMessage:
-#PYLINT:C:Line too long 
             cache = os.path.join(AVATAR_CACHE_FOLDER,os.path.basename(status.user.profile_image_url))
             if not(os.path.exists(cache)):
                 try:
@@ -76,8 +105,6 @@ class KhweeteurWorker(QThread):
         
     def refresh(self):
         print 'Try to refresh'
-        
-        
         current_dt = time.mktime((datetime.datetime.now() - datetime.timedelta(days=14)).timetuple())
         try:
             mlist = []
@@ -103,6 +130,10 @@ class KhweeteurWorker(QThread):
 
         except StandardError,e:
             print e
+            self.emit(SIGNAL("info(const QString&)"),status)
+
+#            KhweeteurNotification().info('Error occurs during refresh.')
+
 
         print 'Refresh ended'
 
@@ -148,6 +179,7 @@ class KhweetsModel(QAbstractListModel):
                         self._new_counter = 0
                         QObject.emit(self, SIGNAL("dataChanged(const QModelIndex&, const QModelIndex &)"), self.createIndex(0,0), self.createIndex(0,len(self._items)))
         except:
+            KhweeteurNotification().info('Wrong cache format. Reinit cache.')
             print 'Wrong cache format'
 
     def serialize(self):
@@ -325,6 +357,7 @@ class KhweeteurWin(QMainWindow):
         self.worker = None
         self.tweetsModel.display_screenname = self.settings.value("displayUser").toBool()
         self.nw = NetworkManager(self.refresh_timeline)
+        self.notifications = KhweeteurNotification()
         self.refresh_timeline()
         self.timer = QTimer()
         self.connect(self.timer, SIGNAL("timeout()"), self.timed_refresh)
@@ -376,7 +409,6 @@ class KhweeteurWin(QMainWindow):
         self.tb_text.setText('@'+user)
 
     def tweet(self):
-        print 'try to tweet'
         try:
             if self.settings.value("login").toString()!='':         
                 api = twitter.Api(username=self.settings.value("login").toString(), password=self.settings.value("password").toString())
@@ -384,14 +416,14 @@ class KhweeteurWin(QMainWindow):
                 status = api.PostUpdate(self.tb_text.text())
                 self.tb_text.setText('')
         except StandardError,e:
-            #FIXME : Display error to user instead of silent fail
-            printe
+            self.notifications.warn('Send tweet failed')
+            print e
 
     def refreshEnded(self):
         self.tweetsModel.serialize()
         counter=self.tweetsModel.getNewAndReset()
-        if (counter>0) and (self.settings.value('useNotification').toBool()):
-            KhweeteurNotification().send('Khweeteur',str(counter)+' new tweet(s)',count=counter)
+        if (counter>0) and (self.settings.value('useNotification').toBool()) and not (self.isActiveWindow()):
+            self.notifications.notify('Khweeteur',str(counter)+' new tweet(s)',count=counter)
         self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator,False)
 
     def do_refresh_now(self):
@@ -399,6 +431,8 @@ class KhweeteurWin(QMainWindow):
         self.worker = KhweeteurWorker()
         self.connect(self.worker, SIGNAL("newStatus(PyQt_PyObject)"), self.tweetsModel.addStatus)
         self.connect(self.worker, SIGNAL("finished()"), self.refreshEnded)
+#        self.notifications.connect(self.worker, SIGNAL('notify(const QString&,const QString&, const int&'), self.notifications.notify)
+        self.notifications.connect(self.worker, SIGNAL('info(const QString&)'), self.notifications.info)
         self.worker.start()
         
     def request_refresh(self):
@@ -449,15 +483,23 @@ class KhweeteurWin(QMainWindow):
 
 class Khweeteur(QApplication):
     def __init__(self):
+        
         QApplication.__init__(self,sys.argv)
         self.setOrganizationName("Khertan Software")
         self.setOrganizationDomain("khertan.net")
         self.setApplicationName("Khweeteur")
         self.version = __version__
-        self.run()
 
+        dbus.mainloop.qt.DBusQtMainLoop(set_as_default=True)
+
+        session_bus = dbus.SessionBus()
+        name = dbus.service.BusName("net.khertan.khweeteur", session_bus)
+        self.dbus_object = KhweeteurDBus(session_bus, '/net/khertan/khweeteur')
+        self.run()
+        
     def run(self):
         self.win = KhweeteurWin()
+        self.dbus_object.attach_win(self.win)
         self.win.show()
         sys.exit(self.exec_())
 
